using System;
using System.Drawing;

namespace PrivacyGuard.AnnotationToolkit
{
	/// <summary>
	/// Summary description for ZoomingChar.
	/// </summary>
	public class ZoomingChar : PrivacyGuard.AnnotationToolkit.AnimationChar
	{
		private SolidBrush textBrush;
		private int alpha = 0;
		public const int ALPHAMAX = 100;
		public const int ALPHAMIN = 0;
		public const int SIZEMAX = 800;
		public const int SIZEMIN = 100;
		private int charSize = SIZEMIN;
		public const string FONTNAME = "Tahoma";

		public ZoomingChar(string l, Point o, Point d) : base (l, o, d)
		{
			this.textBrush = new SolidBrush(Color.FromArgb(ALPHAMAX, Color.OrangeRed));
		}

		override public void UpdateAnimation(double d) 
		{
			this.currPos.X = this.destPos.X - 
				(int)((double)(this.destPos.X - this.orgPos.X) *
					  (1.0f - Math.Atan(d)));
			this.currPos.Y = this.destPos.Y - 
				(int)((double)(this.destPos.Y - this.orgPos.Y) *
				(1.0f - Math.Atan(d)));
			this.charSize = SIZEMIN + (int)((double)(SIZEMAX-SIZEMIN)*Math.Atan(d));
			this.alpha = (int)((double)(ALPHAMAX-ALPHAMIN)*(1.0f - Math.Atan(d)));
			this.alpha = this.alpha > ALPHAMIN? this.alpha : ALPHAMIN;
			//this.alpha = ALPHAMAX;
		}

		override public void DisplayAnimation(Graphics g) 
		{
			this.textBrush.Color = Color.FromArgb(this.alpha, Color.OrangeRed);
			RectangleF drawRectF = new RectangleF
				((float)this.currPos.X-charSize, 
				(float)this.currPos.Y-charSize, 
				(float)charSize*2.0f, 
				(float)charSize*2.0f);
			g.DrawString(this.letter, new Font(FONTNAME, this.charSize), this.textBrush, drawRectF, drawFmt);
		}

		override public void UpdateChar(AnimationChar c) 
		{
			this.letter = (c as ZoomingChar).letter;
			Point newOPos = (c as ZoomingChar).orgPos;
			Point newDPos = (c as ZoomingChar).destPos;
			if (!this.orgPos.Equals(newOPos) || !this.destPos.Equals(newDPos))
				this.invalidateChar = true;
			this.orgPos = newOPos;
			this.destPos = newDPos;
		}
	}
}
