using System;
using System.Collections;

namespace PrivacyGuard.BackEndImpl
{
	/// <summary>
	/// Summary description for ReputationEvaluator.
	/// </summary>
	public class ReputationEvaluator
	{
		public class ReputationRecord 
		{
			private string domain;
			public string Domain 
			{
				get 
				{
					return this.domain;
				}
			}
			private string description;
			public string Description 
			{
				get 
				{
					return this.description;
				}
			}
			private double rating;
			public double Rating 
			{
				get 
				{
					return this.rating;
				}
			}
			private bool ssl;
			public bool Ssl
			{
				get 
				{
					return this.ssl;
				}
			}
			private bool reputed;
			public bool Reputed 
			{
				get 
				{
					return this.reputed;
				}
			}
			private string regtime;
			public string Regtime
			{
				get 
				{
					return this.regtime;
				}
			}
			private bool alertTime;
			public bool AlertTime
			{
				get 
				{
					return this.alertTime;
				}
			}
			private string host;
			public string Host 
			{
				get 
				{
					return this.host;
				}
			}
			private string pop;
			public string Pop 
			{
				get 
				{
					return this.pop;
				}
			}
			private bool alertPop;
			public bool AlertPop 
			{
				get 
				{
					return this.alertPop;
				}
			}
			private string[] certs;
			public string[] Certs 
			{
				get 
				{
					return this.certs;
				}
			}
			public ReputationRecord(string d, string desp, double r, bool ssl, bool rd, string rt, bool at, string h, string p, bool ap, string[] certs) 
			{
				this.domain = d;
				this.description = desp;
				this.rating = r;
				this.ssl = ssl;
				this.reputed = rd;
				this.regtime = rt;
				this.alertTime = at;
				this.host = h;
				this.pop = p;
				this.alertPop = ap;
				this.certs = certs;
			}

			public string GetRatingImage() 
			{
				if (this.rating==0.0) 
					return @"C:\VisualStudio\PrivacyGuard\images\stars-0-0.gif";
				if (this.rating==0.5) 
					return @"C:\VisualStudio\PrivacyGuard\images\stars-0-5.gif";
				if (this.rating==3.0) 
					return @"C:\VisualStudio\PrivacyGuard\images\stars-3-0.gif";
				if (this.rating==4.0) 
					return @"C:\VisualStudio\PrivacyGuard\images\stars-4-0.gif";
				if (this.rating==5.0)
					return @"C:\VisualStudio\PrivacyGuard\images\stars-5-0.gif";
				return @"C:\VisualStudio\PrivacyGuard\images\stars-0-0.gif";
			}
			public bool IsTrustedSite() 
			{
				return (this.rating >= 2.0);
			}
		}

		private Hashtable lookupTable;

		public ReputationEvaluator()
		{
			this.lookupTable = new Hashtable();
			InitializeLookupTable();
		}

		private void InitializeLookupTable() 
		{
			this.lookupTable.Add("ae.com", new ReputationRecord("ae.com", "A verified shopping site", 5.0, true, true, "This site has been online since 1999.", false, "This site is hosted in the United States.", "There are 208 sites that link to this site.", false, new string[]{"BizRate"}));
			this.lookupTable.Add("ae-clearance.com", new ReputationRecord("ae-clearance.com", "An unrecognized site with no trust", 0.0, false, false, "This site is only online for 2 days.", true, "This site is hosted in the United States.", "This site is not linked by any site.", true, null));
			this.lookupTable.Add("alloy.com", new ReputationRecord("alloy.com", "A recognized shopping site", 3.0, true, true, "This site has been online since 1999.", false, "This site is hosted in the United States.", "There are 230 sites that link to this site.", false, null));
			this.lookupTable.Add("amazon.com", new ReputationRecord("amazon.com", "A verified shopping site", 5.0, true, true, "This site has been online since 1994.", false, "This site is hosted in the United States.", "There are about 50000 sites that link to this site.", false, new string[]{"BizRate","CNET"}));
			this.lookupTable.Add("amazon-department.com", new ReputationRecord("amazon-department.com", "An unrecognized site with no trust", 0.0, false, false, "This site is only online for 2 days.", true, "This site is hosted in the United States.", "This site is not linked by any site.", true, null));
			this.lookupTable.Add("appleday.net", new ReputationRecord("appleday.net", "An unrecognized site with no trust", 0.0, false, false, "This site is only online for 10 days", true, "This site is hosted in the United States.", "This site is not linked by any site.", true, null));
			this.lookupTable.Add("apple.com", new ReputationRecord("apple.com", "A verified computer company", 5.0, true, true, "This site has been online since 1987.", false, "This site is hosted in the United States.", "There are about 20000 sites that link to this site.", false, new string[]{"TRUSTe"}));
			this.lookupTable.Add("bedbathandbeyond.com", new ReputationRecord("bedbathandbeyond.com", "A verified shopping site", 4.0, true, true, "This site has been online since 1996.", false, "This site is hosted in the United Sites.", "There are 259 sites that link to this site.", false, new string[]{"BizRate"}));
			this.lookupTable.Add("bestbuy.com", new ReputationRecord("bestbuy.com", "A verified shopping site", 5.0, true, true, "This site has been online since 1994.", false, "This site is hosted in the United States.", "There are about 1000 sites that link to this site.", false, new string[]{"BizRate","CNET"}));
			this.lookupTable.Add("ww2.us", new ReputationRecord("ww2.us", "An untrusted web server hosting 351 websites", 0.0, false, false, "This site has been online since 2004.", false, "This site is hosted in Japan.", "This site is not linked by any site.", true, null));
			this.lookupTable.Add("buy.com", new ReputationRecord("buy.com", "A verified shopping site", 5.0, true, true, "This site has been online since 1998.", false, "This site is hosted in the United States.", "There are about 1000 sites that link to this site.", false, new string[]{"BizRate", "CNET"}));
			this.lookupTable.Add("users-buy.com", new ReputationRecord("users-buy.com", "An unrecognized site with no trust", 0.0, false, false, "This site is only online for 10 days", true, "This site is hosted in the United States.", "This site is not linked by any site.", true, null));
			this.lookupTable.Add("compusa.com", new ReputationRecord("compusa.com", "A verified computer retailer site", 5.0, true, true, "This site has been online since 1994.", false, "This site is hosted in the United States.", "There are 733 sites that link to this site.", false, new string[]{"BizRate", "CNET"}));
			this.lookupTable.Add("kasserver.com", new ReputationRecord("kasserver.com", "An unrecognized site with no trust", 0.0, false, false, "This site has been online since 2002.", false, "This site is hosted in Germany.", "This site is not linked by any site.", true, null));
			this.lookupTable.Add("deepdiscountdvd.com", new ReputationRecord("deepdiscountdvd.com", "A verified shopping site", 4.0, true, true, "This site has been online since 2000.", false, "This site is hosted in the United States.", "There are 160 sites that link to this site.", false, new string[]{"ScanAlert"}));
			this.lookupTable.Add("global-update2.com", new ReputationRecord("global-update2.com", "An unrecognized site with no trust", 0.0, false, false, "This site is only online for 10 days.", true, "This site is hosted in the United States.", "This site is not linked by any site.", true, null));
			this.lookupTable.Add("dvdplanet.com", new ReputationRecord("dvdplanet.com", "A verified shopping site", 4.0, true, true, "This site has been online since 1998.", false, "This site is hosted in the United States.", "There are 103 sites that link to this site.", false, new string[]{"BizRate"}));
			this.lookupTable.Add("dvdplanetserviceonline.com", new ReputationRecord("dvdplanetserviceonline.com", "An unrecognized site with no trust", 0.0, false, false, "This site is only online for 10 days.", true, "This site is hosted in the United States.", "This site is not linked by any site.", true, null));
			this.lookupTable.Add("ebgames.com", new ReputationRecord("ebgames.com", "A verified shopping site", 5.0, true, true, "This site has been online since 1999.", false, "This site is hosted in the United States.", "There are 589 sites that link to this site.", false, new string[]{"ScanAlert", "CNET"}));
			this.lookupTable.Add("ssl-ebgames.com", new ReputationRecord("ssl-ebgames.com", "An unrecognized site with no trust", 0.0, false, false, "This site is only online for 1 day.", true, "This site is hosted in the United States.", "This site is not linked by any site.", true, null));
			this.lookupTable.Add("hollywoodvideo.com", new ReputationRecord("hollywoodvideo.com", "A recognized retailer site", 3.0, true, true, "This site has been online since 1999.", false, "This site is hosted in the United States.", "There are 126 sites that link to this site.", false, null));
			this.lookupTable.Add("hsn.com", new ReputationRecord("hsn.com", "A verified shopping site", 5.0, true, true, "This site has been online since 1997.", false, "This site is hosted in the United States.", "There are 298 sites that link to this site.", false, new string[]{"BizRate"}));
			this.lookupTable.Add("hsn-online.net", new ReputationRecord("hsn-online.net", "An unrecognized site with no trust", 0.0, false, false, "This site is only online for 10 days.", true, "This site is hosted in the United States.", "This site is not linked by any site.", true, null));
			this.lookupTable.Add("kbtoys-hsd.net", new ReputationRecord("kbtoys-hsd.net", "An unrecognized site with no trust", 0.0, false, false, "This site has been online since 1999.", false, "This site is hosted in the United States.", "This site is not linked by any site.", true, null));
			this.lookupTable.Add("kbtoys.com", new ReputationRecord("kbtoys.com", "A verified shopping site", 5.0, true, true, "This site has been online since 1995.", false, "This site is hosted in the United States.", "There are 381 sites that link to this site.", false, new string[]{"BizRate", "GeoTrust"}));
			this.lookupTable.Add("kmart.com", new ReputationRecord("kmart.com", "A verified shopping site", 5.0, true, true, "This site has been online since 1994.", false, "This site is hosted in the United States.", "There are 679 sites that link to this site.", false, new string[]{"BizRate"}));
			this.lookupTable.Add("sales-kmart-online.com", new ReputationRecord("sales-kmart-online.com", "An unrecognized site with no trust", 0.0, false, false, "This site is only online for 2 days.", true, "This site is hosted in the United States.", "This site is not linked by any site.", true, null));
			this.lookupTable.Add("macys.com", new ReputationRecord("macys.com", "A recognized shopping site", 3.0, true, true, "This site has been online since 1994.", false, "This site is hosted in the United States.", "There are 462 sites that link to this site.", false, null));
			this.lookupTable.Add("mymacys.com", new ReputationRecord("mymacys.com", "An unrecognized site with no trust", 0.0, false, false, "This site is only online for 1 day.", true, "This site is hosted in the United States.", "This site is not linked by any site.", true, null));
			this.lookupTable.Add("officemax.com", new ReputationRecord("officemax.com", "A verified shopping site", 5.0, true, true, "This site has been online since 1995.", false, "This site is hosted in the United States.", "There are 452 sites that link to this site.", false, new string[]{"BizRate"}));
			this.lookupTable.Add("paypal.com", new ReputationRecord("paypal.com", "A verified business site", 5.0, true, true, "This site has been online since 1999.", false, "This site is hosted in the United States.", "There are about 6000 sites that link to this site.", false, new string[]{"TRUSTe"}));
			this.lookupTable.Add("pcconnection.com", new ReputationRecord("pcconnection.com", "A verified business site", 5.0, true, true, "This site has been online since 1994.", false, "This site is hosted in the United States.", "There are 233 sites that link to this site.", false, new string[]{"BizRate", "CNET"}));
			this.lookupTable.Add("pcconnection-user-online.com", new ReputationRecord("pcconnection-user-online.com", "An unrecognized site with no trust", 0.0, false, false, "This site is only online for 1 day.", true, "This site is hosted in the United States.", "This site is not linked by any site.", true, null));
			this.lookupTable.Add("staples.com", new ReputationRecord("staples.com", "A verified shopping site", 5.0, true, true, "This site has been online since 1995.", false, "This site is hosted in the United States.", "There are 588 sites that link to this site.", false, new string[]{"BizRate", "CNET"}));
			this.lookupTable.Add("special-deal-staples.com", new ReputationRecord("special-deal-staples.com", "An unrecognized site with no trust", 0.0, false, false, "This site is only online for 1 day.", true, "This site is hosted in the United States.", "This site is not linked by any site.", true, null));
			this.lookupTable.Add("target.com", new ReputationRecord("target.com", "A verified shopping site", 5.0, true, true, "This site has been online since 1997.", false, "This site is hosted in the United States.", "There are about 1000 sites that link to this site.", false, new string[]{"BizRate"}));
			this.lookupTable.Add("my-target-store.com", new ReputationRecord("my-target-store.com", "An unrecognized site with no trust", 0.0, false, false, "This site is only online for 1 day.", true, "This site is hosted in the United States.", "This site is not linked by any site.", true, null));
			this.lookupTable.Add("travelocity.com", new ReputationRecord("travelocity.com", "A recognized recreation site", 4.0, true, true, "This site has been online since 1997.", false, "This site is hosted in the United States.", "There are about 2000 sites that link to this site.", false, null));
			this.lookupTable.Add("zaga-zaga.us", new ReputationRecord("zaga-zaga.us", "An untrusted web server hosting 103 websites", 0.0, false, false, "This site has been online since 2005.", false, "This site is hosted in China.", "This site is not linked by any site.", true, null));
			this.lookupTable.Add("walmart.com", new ReputationRecord("walmart.com", "A verified shopping site", 5.0, true, true, "This site has been online since 1995.", false, "This site is hosted in the United States.", "There are about 2000 sites that link to this site.", false, new string[]{"BizRate", "CNET"}));
			this.lookupTable.Add("yahoo.com", new ReputationRecord("yahoo.com", "A verified computer-Internet site", 5.0, true, true, "This site has been online since 1995.", false, "This site is hosted in the United States.", "There are about 30000 sites that link to this site.", false, new string[]{"BizRate", "TRUSTe", "GeoTrust"}));
			this.lookupTable.Add("yahoo-inc-shopping.com", new ReputationRecord("yahoo-inc-shopping.com", "An unrecognized site with no trust", 0.0, false, false, "This site is only online for 1 day.", true, "This site is hosted in the United States.", "This site is not linked by any site.", true, null));
			this.lookupTable.Add("defensivewebbrowsing.org", new ReputationRecord("defensivewebbrowsing.org", "A verified computer-Internet site", 5.0, true, true, "This site has been online since 2002.", false, "This site is hosted in the United States.", "There are about 1000 sites that link to this site.", false, new string[]{"BizRate", "TRUSTe", "GeoTrust", "CNET"}));
			this.lookupTable.Add("ebay.com", new ReputationRecord("ebay.com", "A verified shopping site", 5.0, true, true, "This site has been online since 1995.", false, "This site is hosted in the United States.", "There are about 8000 sites that link to this site.", false, new string[]{"BizRate", "TRUSTe"}));
			this.lookupTable.Add("expedia.com", new ReputationRecord("expedia.com", "A verified recreation site", 5.0, true, true, "This site has been online since 1995.", false, "This site is hosted in the United States.", "There are about 3000 sites that link to this site.", false, new string[]{"BizRate", "TRUSTe", "GeoTrust"}));
			this.lookupTable.Add("bloomingdales.com", new ReputationRecord("bloomingdales.com", "A verified shopping site", 5.0, true, true, "This site has been online since 1995.", false, "This site is hosted in the United States.", "There are 272 sites that link to this site.", false, new string[]{"BizRate"}));
			this.lookupTable.Add("member-update.info", new ReputationRecord("member-update.info", "An unrecognized site with no trust", 0.0, false, false, "This site is only online for 10 days.", true, "This site is hosted in the United States.", "This site is not linked by any site.", true, null));
			this.lookupTable.Add("radioshack.com", new ReputationRecord("radioshack.com", "A verified shopping site", 5.0, true, true, "This site has been online since 1996.", false, "This site is hosted in the United States.", "There are 716 sites that link to this site.", false, new string[]{"BizRate", "TRUSTe", "GeoTrust"}));
			this.lookupTable.Add("no-ip.info", new ReputationRecord("no-ip.info", "An untrusted web server hosting 38 websites", 0.0, false, false, "This site has been online since 2001.", false, "This site is hosted in the United States.", "This site is not linked by any site.", true, null));
			this.lookupTable.Add("nc-ida.org", new ReputationRecord("nc-ida.org", "An untrusted web server hosting 8138 websites", 0.0, false, false, "This site has been online since 2004.", false, "This site is hosted in the United States.", "This site is not linked by any site.", true, null));
			this.lookupTable.Add("comsec-data.dk", new ReputationRecord("comsec-data.dk", "An unrecognized site with no trust", 0.0, false, false, "This site has been online since 2002.", false, "This site is hosted in Denmark.", "This site is not linked by any site.", true, null));
			this.lookupTable.Add("google.com", new ReputationRecord("google.com", "A verified computer-Internet site", 5.0, true, true, "This site has been online since 1997.", false, "This site is hosted in the United States.", "There are about 50000 sites that link to this site.", false, new string[]{"BizRate"}));
		}

		public ReputationRecord GetReputationRecord(string domain) 
		{
			ReputationRecord rr = (ReputationRecord) this.lookupTable[domain];
			if (rr == null)
				rr = new ReputationRecord(domain, "An unrecognized site with no trust", 0.0, false, false, "No information about this site", true, "", "", true, null);
			return rr;
		}

		public bool IsTrustedSite(string domain) 
		{
			double rating;
			ReputationRecord rr = (ReputationRecord) this.lookupTable[domain];
			if (rr != null)
				rating = rr.Rating;
			else
				rating = 0.0;

			return (rating >= 2.0);
		}

		public string GetDomainDescription(string domain) 
		{
			ReputationRecord rr = (ReputationRecord) this.lookupTable[domain];
			if (rr != null)
				return rr.Description;
			else
				return "An unrecognized site with no trust";
		}

		public static void SortReputationRecords(ref ArrayList record) 
		{
			NameSorter ns = new NameSorter();
			record.Sort(ns);
//			ArrayList trusted = new ArrayList();
//			ArrayList untrusted = new ArrayList();
//			IEnumerator ienum = record.GetEnumerator();
//			while (ienum.MoveNext()) 
//			{
//				ReputationRecord rr = (ReputationRecord) ienum.Current;
//				if (rr.IsTrustedSite())
//					trusted.Add(rr);
//				else
//					untrusted.Add(rr);
//			}
//			NameSorter ns = new NameSorter();
//			trusted.Sort(ns);
//			untrusted.Sort(ns);
//			record = new ArrayList(trusted);
//			record.AddRange(untrusted);
//			return trusted.Count;
		}

		class NameSorter : IComparer 
		{
			int IComparer.Compare(object x, object y) 
			{
				if (x is ReputationRecord && y is ReputationRecord) 
				{
					return String.Compare(
						(x as ReputationRecord).Domain,
						(y as ReputationRecord).Domain,
						false);
				} else
					return 0;
			}
		}
	}
}
