using System;
using System.Collections;
using mshtml;

namespace PrivacyGuard.BackEndImpl
{
	/// <summary>
	/// Summary description for WebpageAnalyzer.
	/// </summary>
	public class WebpageAnalyzer
	{
		private Hashtable lookupTable;

		public class HardcodedInput 
		{
			protected string name;
			public string Name 
			{
				get 
				{
					return this.name;
				}
			}
			protected WebWallet.ConstantAttributes.INPUTTYPE inputtype;
			public WebWallet.ConstantAttributes.INPUTTYPE Inputtype 
			{
				get 
				{
					return this.inputtype;
				}
			}
			protected WebWallet.ConstantAttributes.ATTRIBUTE attribute;
			public WebWallet.ConstantAttributes.ATTRIBUTE Attribute 
			{
				get 
				{
					return this.attribute;
				}
			}
			public HardcodedInput(string n, WebWallet.ConstantAttributes.INPUTTYPE it) 
			{
				this.name = n;
				this.inputtype = it;
				this.attribute = WebWallet.ConstantAttributes.ATTRIBUTE.NONE;
			}
			public HardcodedInput(string n, WebWallet.ConstantAttributes.INPUTTYPE it, WebWallet.ConstantAttributes.ATTRIBUTE attr) :
				this (n, it)
			{
				this.attribute = attr;
			}
		}

		public class HardcodedPasswordInput : HardcodedInput 
		{
			protected string authScope = BackEndImpl.SecureStorage.DEFAULTSCOPE;
			public string AuthScope 
			{
				get 
				{
					return this.authScope;
				}
			}
			public HardcodedPasswordInput(string n, WebWallet.ConstantAttributes.INPUTTYPE it) : base (n, it) 
			{
			}
			public HardcodedPasswordInput(string n, WebWallet.ConstantAttributes.INPUTTYPE it, WebWallet.ConstantAttributes.ATTRIBUTE attr) : base (n, it, attr) 
			{
			}
			public HardcodedPasswordInput(string n, string s, WebWallet.ConstantAttributes.INPUTTYPE it) : base (n, it) 
			{
				this.authScope = s;
			}
			public HardcodedPasswordInput(string n, string s, WebWallet.ConstantAttributes.INPUTTYPE it, WebWallet.ConstantAttributes.ATTRIBUTE attr) : base (n, it, attr) 
			{
				this.authScope = s;
			}
		}

		public WebpageAnalyzer()
		{
			this.lookupTable = new Hashtable();
			InitializeLookupTable();
		}

		private void InitializeLookupTable() 
		{
			// amazon.com part
			string domain = "amazon.com";
			ArrayList domainlist = new ArrayList();
			ArrayList domainentry = new ArrayList();
			domainentry.Add(new HardcodedInput("email", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_ID));
			domainentry.Add(new HardcodedPasswordInput("password", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_PASSWORD));
			domainlist.Add(domainentry);
			lookupTable.Add(domain, domainlist);
			// bestbuy.com part
			domain = "bestbuy.com";
			domainlist = new ArrayList();
			domainentry = new ArrayList();
			domainentry.Add(new HardcodedInput("TxtEmail", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_ID));
			domainentry.Add(new HardcodedPasswordInput("PwdPassword", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_PASSWORD));
			domainlist.Add(domainentry);
			lookupTable.Add(domain, domainlist);
			// ae.com part
			domain = "ae.com";
			domainlist = new ArrayList();
			domainentry = new ArrayList();
			domainentry.Add(new HardcodedInput("logonUsername", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_ID));
			domainentry.Add(new HardcodedPasswordInput("logonPassword", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_PASSWORD));
			domainlist.Add(domainentry);
			domainentry = new ArrayList();
			domainentry.Add(new HardcodedInput("email", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_ID));
			domainentry.Add(new HardcodedPasswordInput("pass", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_PASSWORD));
			domainlist.Add(domainentry);
			domainentry = new ArrayList();
			domainentry.Add(new HardcodedInput("username", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_ID));
			domainentry.Add(new HardcodedPasswordInput("password", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_PASSWORD));
			domainlist.Add(domainentry);
			lookupTable.Add(domain, domainlist);
			// target.com part
			domain = "target.com";
			domainlist = new ArrayList();
			domainentry = new ArrayList();
			domainentry.Add(new HardcodedInput("email", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_ID));
			domainentry.Add(new HardcodedPasswordInput("password", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_PASSWORD));
			domainlist.Add(domainentry);
			lookupTable.Add(domain, domainlist);
			// compusa.com part
			domain = "compusa.com";
			domainlist = new ArrayList();
			domainentry = new ArrayList();
			domainentry.Add(new HardcodedInput("frm_email", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_ID));
			domainentry.Add(new HardcodedPasswordInput("frm_password", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_PASSWORD));
			domainlist.Add(domainentry);
			domainentry = new ArrayList();
			domainentry.Add(new HardcodedInput("frm_email_address", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_ID));
			domainentry.Add(new HardcodedPasswordInput("frm_passwordNew1", 
				WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_PASSWORD,
				WebWallet.ConstantAttributes.ATTRIBUTE.NEW));
			domainentry.Add(new HardcodedPasswordInput("frm_passwordNew2", 
				WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_PASSWORD,
				WebWallet.ConstantAttributes.ATTRIBUTE.NEW));
			domainlist.Add(domainentry);
			lookupTable.Add(domain, domainlist);
			// radioshack.com part
			domain = "radioshack.com";
			domainlist = new ArrayList();
			domainentry = new ArrayList();
			domainentry.Add(new HardcodedInput("email", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_ID));
			domainentry.Add(new HardcodedPasswordInput("password", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_PASSWORD));
			domainlist.Add(domainentry);
			domainentry = new ArrayList();
			domainentry.Add(new HardcodedInput("email", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_ID));
			domainentry.Add(new HardcodedPasswordInput("newPassword", 
				WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_PASSWORD,
				WebWallet.ConstantAttributes.ATTRIBUTE.NEW));
			domainentry.Add(new HardcodedPasswordInput("newPassword2", 
				WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_PASSWORD,
				WebWallet.ConstantAttributes.ATTRIBUTE.NEW));
			domainlist.Add(domainentry);
			lookupTable.Add(domain, domainlist);
			/// buy.com part
			domain = "buy.com";
			domainlist = new ArrayList();
			domainentry = new ArrayList();
			domainentry.Add(new HardcodedInput("shopper_name", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_ID));
			domainentry.Add(new HardcodedPasswordInput("shopper_password", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_PASSWORD));
			domainlist.Add(domainentry);
			lookupTable.Add(domain, domainlist);
			/// deepdiscountdvd.com part
			domain = "deepdiscountdvd.com";
			domainlist = new ArrayList();
			domainentry = new ArrayList();
			domainentry.Add(new HardcodedInput("email", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_ID));
			domainentry.Add(new HardcodedPasswordInput("password", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_PASSWORD));
			domainlist.Add(domainentry);
			lookupTable.Add(domain, domainlist);
			/// ebgames.com part
			domain = "ebgames.com";
			domainlist = new ArrayList();
			domainentry = new ArrayList();
			domainentry.Add(new HardcodedInput("BillTo_Email", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_ID));
			domainentry.Add(new HardcodedPasswordInput("Password", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_PASSWORD));
			domainlist.Add(domainentry);
			lookupTable.Add(domain, domainlist);
			/// hsn.com part
			domain = "hsn.com";
			domainlist = new ArrayList();
			domainentry = new ArrayList();
			domainentry.Add(new HardcodedInput(@"_PhoneEmail", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_ID));
			domainentry.Add(new HardcodedPasswordInput("pwd", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_PASSWORD));
			domainlist.Add(domainentry);
			lookupTable.Add(domain, domainlist);
			/// kmart.com part
			domain = "kmart.com";
			domainlist = new ArrayList();
			domainentry = new ArrayList();
			domainentry.Add(new HardcodedInput("emailAddress", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_ID));
			domainentry.Add(new HardcodedPasswordInput("password", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_PASSWORD));
			domainlist.Add(domainentry);
			lookupTable.Add(domain, domainlist);
			/// macys.com part
			domain = "macys.com";
			domainlist = new ArrayList();
			domainentry = new ArrayList();
			domainentry.Add(new HardcodedInput("EmailAddress", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_ID));
			domainentry.Add(new HardcodedPasswordInput("Password", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_PASSWORD));
			domainlist.Add(domainentry);
			lookupTable.Add(domain, domainlist);
			/// travelocity.com part
			domain = "travelocity.com";
			domainlist = new ArrayList();
			domainentry = new ArrayList();
			domainentry.Add(new HardcodedInput(@"es_alias", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_ID));
			domainentry.Add(new HardcodedPasswordInput(@"es_passwd", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_PASSWORD));
			domainlist.Add(domainentry);
			lookupTable.Add(domain, domainlist);
			/// yahoo.com part
			domain = "yahoo.com";
			domainlist = new ArrayList();
			domainentry = new ArrayList();
			domainentry.Add(new HardcodedInput("login", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_ID));
			domainentry.Add(new HardcodedPasswordInput("passwd", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_PASSWORD));
			domainlist.Add(domainentry);
			lookupTable.Add(domain, domainlist);
			/// apple.com part
			domain = "apple.com";
			domainlist = new ArrayList();
			domainentry = new ArrayList();
			domainentry.Add(new HardcodedInput("theAccountName", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_ID));
			domainentry.Add(new HardcodedPasswordInput("theAccountPW", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_PASSWORD));
			domainlist.Add(domainentry);
			lookupTable.Add(domain, domainlist);
			/// dvdplanet.com part
			domain = "dvdplanet.com";
			domainlist = new ArrayList();
			domainentry = new ArrayList();
			domainentry.Add(new HardcodedInput("loginemail", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_ID));
			domainentry.Add(new HardcodedPasswordInput("loginpassword", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_PASSWORD));
			domainlist.Add(domainentry);
			lookupTable.Add(domain, domainlist);
			/// kbtoys.com part
			domain = "kbtoys.com";
			domainlist = new ArrayList();
			domainentry = new ArrayList();
			domainentry.Add(new HardcodedInput("Aemail", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_ID));
			domainentry.Add(new HardcodedPasswordInput("password", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_PASSWORD));
			domainlist.Add(domainentry);
			lookupTable.Add(domain, domainlist);
			/// pcconnection.com part
			domain = "pcconnection.com";
			domainlist = new ArrayList();
			domainentry = new ArrayList();
			domainentry.Add(new HardcodedInput(@"ReturningCustomerLogin1:UsernameCtrl", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_ID));
			domainentry.Add(new HardcodedPasswordInput(@"ReturningCustomerLogin1:PasswordCtrl", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_PASSWORD));
			domainlist.Add(domainentry);
			lookupTable.Add(domain, domainlist);
			/// staples.com part
			domain = "staples.com";
			domainlist = new ArrayList();
			domainentry = new ArrayList();
			domainentry.Add(new HardcodedInput("logonId", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_ID));
			domainentry.Add(new HardcodedPasswordInput("staplesPassword", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_PASSWORD));
			domainlist.Add(domainentry);
			lookupTable.Add(domain, domainlist);
			/// bloomingdales.com part
			domain = "bloomingdales.com";
			domainlist = new ArrayList();
			domainentry = new ArrayList();
			domainentry.Add(new HardcodedInput("EmailAddress", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_ID));
			domainentry.Add(new HardcodedPasswordInput("Password", WebWallet.ConstantAttributes.INPUTTYPE.LOGIN_PASSWORD));
			domainlist.Add(domainentry);
			lookupTable.Add(domain, domainlist);
		}

		public bool IsSubmitElement(string domainname, IHTMLElement elem) 
		{
			if (domainname.Equals("staples.com"))
			{
				string s = elem.innerText;
				if (s!=null) 
				{
					if (s.Equals("Log In"))
						return true;
				}
			}
			return false;
		}
		public bool IsSubmitButton(string domainname, IHTMLInputElement button) 
		{
			if (domainname.Equals("ae.com")) 
			{
				string s = button.name;
				if (s != null) 
				{
					if (s.Equals("signIn"))
						return true;
					if (s.Equals(@"/atg/userprofiling/ProfileFormHandler.login"))
						return true;
				}
			}
			if (domainname.Equals("amazon.com")) 
			{
				string s = button.value;
				if (s != null) 
				{
					if (s.Equals("Continue"))
						return true;
				}
			}
			if (domainname.Equals("apple.com")) 
			{
				string s = button.name;
				if (s != null) 
				{
					if (s.Equals(@"1.Continue"))
						return true;
				}
			}
			if (domainname.Equals("bestbuy.com")) 
			{
				string s = button.value;
				if (s != null)
				{
					if (s.Equals("Sign In"))
						return true;
				}
			}
			if (domainname.Equals("bloomingdales.com")) 
			{
				string s = button.value;
				if (s!=null) 
				{
					if (s.Equals("SIGNINSIGNIN_BUTTON"))
						return true;
				}
			}
			if (domainname.Equals("buy.com")) 
			{
				string s = button.src;
				if (s != null) 
				{
					if (s.EndsWith("btn_text_login2.gif"))
						return true;
					if (s.EndsWith("button_submit2.gif"))
						return true;
				}
			}
			if (domainname.Equals("radioshack.com")) 
			{
				string s = button.src;
				if (s != null) 
				{
					if (s.EndsWith("cart2_signin.gif"))
						return true;
				}
			}
			if (domainname.Equals("compusa.com")) 
			{
				string s = button.value;
				if (s!=null) 
				{
					if (s.Equals("Log In"))
						return true;
				}
			}
			if (domainname.Equals("deepdiscountdvd.com")) 
			{
				string s = button.value;
				if (s!=null) 
				{
					if (s.Equals("Log-In"))
						return true;
				}
			}
			if (domainname.Equals("dvdplanet.com")) 
			{
				string s = button.value;
				if (s != null) 
				{
					if (s.Equals("Login"))
						return true;
				}
			}
			if (domainname.Equals("ebgames.com")) 
			{
				string s = button.value;
				if (s!= null) 
				{
					if (s.Equals("Log in and continue."))
						return true;
				}
			}
//			if (domainname.Equals("gifts.com")) 
//			{
//				string s = button.src;
//				if (s != null) 
//				{
//					if (s.EndsWith("but-signIn.gif"))
//						return true;
//				}
//			}
			if (domainname.Equals("hsn.com")) 
			{
				string s = button.src;
				if (s != null) 
				{
					if (s.EndsWith("b-submit-button.gif"))
						return true;
				}
			}
			if (domainname.Equals("kbtoys.com")) 
			{
				string s = button.name;
				if (s!=null) 
				{
					if (s.Equals("searchWishlist"))
						return true;
				}
			}
			if (domainname.Equals("kmart.com")) 
			{
				string s = button.value;
				if (s!=null) 
				{
					if (s.Equals("login"))
						return true;
				}
			}
			if (domainname.Equals("macys.com")) 
			{
				string s = button.value;
				if (s!=null) 
				{
					if (s.Equals("SIGNINSIGNIN_BUTTON"))
						return true;
				}
			}
			if (domainname.Equals("pcconnection.com")) 
			{
				string s = button.name;
				if (s!=null) 
				{
					if (s.Equals(@"ReturningCustomerLogin1:Login"))
						return true;
				}
			}
			if (domainname.Equals("target.com")) 
			{
				string s = button.name;
				if (s!=null) 
				{
					if (s.Equals("submit_sign_in"))
						return true;
				}
			}
			if (domainname.Equals("travelocity.com"))
			{
				string s = button.src;
				if (s!=null) 
				{
					if (s.EndsWith("bt_a1_login.gif"))
						return true;
				}
			}
			if (domainname.Equals("yahoo.com")) 
			{
				string s = button.value;
				if (s!=null) 
				{
					if (s.Equals("Sign In"))
						return true;
				}
			}
			return false;
		}
		public ArrayList HardcodedAnalyze(IHTMLDocument2 doc) 
		{
			ArrayList groups = new ArrayList();

			IHTMLElementCollection c = (IHTMLElementCollection) doc.all.tags("INPUT");
			string domainname = PrivacyGuardUtils.IEUtil.GetDomainName(doc.url);
			if (domainname == null)
				return groups;

			ArrayList potentialHolders = this.lookupTable[domainname] as ArrayList;
			if (potentialHolders == null)
				return groups;

			IEnumerator holderenum = potentialHolders.GetEnumerator();
			while (holderenum.MoveNext()) 
			{
				WebWallet.InlineProtectionToolkit.ProtectedGroup.ProtectedHolder currentReceiver = 
					new WebWallet.InlineProtectionToolkit.ProtectedGroup.ProtectedHolder();

				ArrayList currentHolder = holderenum.Current as ArrayList;
				IEnumerator entryenum = currentHolder.GetEnumerator();
				if (entryenum.MoveNext()) 
				{
					HardcodedInput currentInput = entryenum.Current as HardcodedInput;
					for (int i = 0; i < c.length; i++) 
					{
						IHTMLInputElement inel = c.item(i, i) as IHTMLInputElement;
						if (inel.name != null && inel.name.Equals(currentInput.Name)) 
						{
							string scope = BackEndImpl.SecureStorage.DEFAULTSCOPE;
							if (currentInput is HardcodedPasswordInput) 
								scope = (currentInput as HardcodedPasswordInput).AuthScope;

							currentReceiver.addInput(
								new WebWallet.InlineProtectionToolkit.ProtectedGroup.ProtectedInput(inel, currentInput.Inputtype, currentInput.Attribute), scope);
							if (entryenum.MoveNext()) 
							{
								currentInput = entryenum.Current as HardcodedInput;
							} 
							else 
							{
								currentReceiver.setDomain(PrivacyGuardUtils.IEUtil.GetDomainName(doc.url));
								groups.Add(currentReceiver);

								if (holderenum.MoveNext()) 
								{
									currentReceiver = 
										new WebWallet.InlineProtectionToolkit.ProtectedGroup.ProtectedHolder();
									currentHolder = holderenum.Current as ArrayList;
									entryenum = currentHolder.GetEnumerator();
									if (entryenum.MoveNext())
										currentInput = entryenum.Current as HardcodedInput;
								}
							}
						}
					}
				}
			}
			return groups;
		}
	}
}
