using System;
using System.Drawing;
using System.Collections;
using mshtml;

namespace PrivacyGuard.WebWallet.InlineProtectionToolkit
{
	/// <summary>
	/// Summary description for ProtectedInputs.
	/// </summary>
	public class ProtectedGroup
	{
		public const string READYONLYCOLOR = "#808080";
		protected IEBrowser browser;
		protected BackEndImpl.WebpageAnalyzer analyzer;
		protected ArrayList groups;
		protected ProtectedHolder focus;

		public class ProtectedHolder 
		{
			public const int PADDING = 5;
			public const string FULLALPHA = "alpha(opacity=100)";
			public const string INIALPHA = "alpha(opacity=30)";
			//public const string BORDER = "#ff4500 2px solid";
			public const string BORDER = "#008000 2px solid";
			public const string TOOLTIP = "Press the F2 key to open the Web Wallet";
			
			//private string image_uri = @"file:///C:/VisualStudio/PrivacyGuard/images/protect-smaller.png";
			private string image_uri = @"https://people.csail.mit.edu/minwu/protect-smaller.png";
			private string image_elem;

			private WebWallet.InlineProtectionToolkit.SensitiveData.SensitiveData sdata;
			public WebWallet.InlineProtectionToolkit.SensitiveData.SensitiveData Sdata 
			{
				get 
				{
					return this.sdata;
				}
				set 
				{
					this.sdata = value;
				}
			}

			ArrayList items;
			public ArrayList Items 
			{
				get 
				{
					return this.items;
				}
			}
			Rectangle boundary;
			bool inside = false;
			public Rectangle Boundary 
			{
				get 
				{
					return this.boundary;
				}
			}
			IHTMLElement divElem;
			public IHTMLElement DivElem 
			{
				get 
				{
					return this.divElem;
				}
			}
			ArrayList imageIcons;
			private bool alwaysHighlight = false;

			public ProtectedHolder() 
			{
				this.items = new ArrayList();
				this.imageIcons = new ArrayList();
				this.image_elem = "<img src=\"" + image_uri + "\" style=\"filter:" + INIALPHA + "\"></img>";
			}

			public void setDomain(string domain) 
			{
				if (this.sdata is WebWallet.InlineProtectionToolkit.SensitiveData.LoginToken) 
					(this.sdata as WebWallet.InlineProtectionToolkit.SensitiveData.LoginToken).Domain = domain;
			}

			public void addInput(ProtectedInput pi, string scope) 
			{
				ConstantAttributes.INPUTTYPE t = pi.SensitiveType;
				if (this.sdata == null) 
				{
					if (t.Equals(ConstantAttributes.INPUTTYPE.LOGIN_ID) ||
						t.Equals(ConstantAttributes.INPUTTYPE.LOGIN_PASSWORD))
						this.sdata = new WebWallet.InlineProtectionToolkit.SensitiveData.LoginToken();
				}
				if (this.sdata is WebWallet.InlineProtectionToolkit.SensitiveData.LoginToken) 
				{
					if (t.Equals(ConstantAttributes.INPUTTYPE.LOGIN_ID))
						(this.sdata as WebWallet.InlineProtectionToolkit.SensitiveData.LoginToken).AddId(pi.Elem as IHTMLElement);
					if (t.Equals(ConstantAttributes.INPUTTYPE.LOGIN_PASSWORD))
						(this.sdata as WebWallet.InlineProtectionToolkit.SensitiveData.LoginToken).AddPassword(pi.Elem as IHTMLElement, scope);
				}
				this.items.Add(pi);
			}
			public void InlineAnnotation(IHTMLDocument2 doc) 
			{
//				int offsetTop = (doc.body as IHTMLElement2).scrollTop;
//				int offsetLeft = (doc.body as IHTMLElement2).scrollLeft;

				int minimalLeft = int.MaxValue;
				int maximalRight = int.MinValue;
				int minimalTop = int.MaxValue;
				int maximalBottom = int.MinValue;

				IEnumerator ienum = items.GetEnumerator();
				while (ienum.MoveNext()) 
				{
					ProtectedInput input = ienum.Current as ProtectedInput;
					IHTMLInputElement inel = input.Elem;
					IHTMLElement imgel = doc.createElement(this.image_elem);
					(inel as IHTMLElement2).insertAdjacentElement("beforeBegin", imgel);
					this.imageIcons.Add(imgel);
					Rectangle inputRect = this.GetInlinePosition(inel as IHTMLElement);
					Rectangle imageRect = this.GetInlinePosition(imgel);
					int minleft = Math.Min(inputRect.Left, imageRect.Left);
					minimalLeft = minimalLeft > minleft ? minleft : minimalLeft;
					int maxright = Math.Max(inputRect.Right, imageRect.Right);
					maximalRight = maximalRight < maxright ? maxright : maximalRight;
					int mintop = Math.Min(inputRect.Top, imageRect.Top);
					minimalTop = minimalTop > mintop ? mintop : minimalTop;
					int maxbottom = Math.Max(inputRect.Bottom, imageRect.Bottom);
					maximalBottom = maximalBottom < maxbottom ? maxbottom : maximalBottom;

//					IHTMLRect relInputRect = (inel as IHTMLElement2).getBoundingClientRect();
//					IHTMLRect relImgRect = (imgel as IHTMLElement2).getBoundingClientRect();
//
//					int minleft = Math.Min(relInputRect.left, relImgRect.left);
//					minimalLeft = minimalLeft > minleft ? minleft : minimalLeft;
//					int maxright = Math.Max(relInputRect.right, relImgRect.right);
//					maximalRight = maximalRight < maxright ? maxright : maximalRight;
//					int mintop = Math.Min(relInputRect.top, relImgRect.top);
//					minimalTop = minimalTop > mintop ? mintop : minimalTop;
//					int maxbottom = Math.Max(relInputRect.bottom, relImgRect.bottom);
//					maximalBottom = maximalBottom < maxbottom ? maxbottom : maximalBottom;

					if (input.SensitiveType.Equals(ConstantAttributes.INPUTTYPE.LOGIN_PASSWORD)) 
					{
						inel.readOnly = true;
						(inel as IHTMLElement).style.backgroundColor = ProtectedGroup.READYONLYCOLOR;
					}
				}
				int divX = minimalLeft - PADDING;
				divX = divX > 0 ? divX : 0;
				int divY = minimalTop - PADDING;
				divY = divY > 0 ? divY : 0;
				int divW = maximalRight + PADDING - divX;
				int divH = maximalBottom + PADDING - divY;
//				divX += offsetTop;
//				divY += offsetLeft;
				this.boundary = new Rectangle(divX, divY, divW, divH);
				string divXStr = divX.ToString() + "px";
				string divYStr = divY.ToString() + "px";
				string divWStr = divW.ToString() + "px";
				string divHStr = divH.ToString() + "px";
				string divElemStr = "<div style=\"position:absolute; left:" + divXStr + "; top:" + divYStr + 
					"; width:" + divWStr + "; height:" + divHStr + 
					"; border: " + BORDER + "; filter: " + INIALPHA + "; z-index:101\"></div>";
				IHTMLElement divElem = doc.createElement(divElemStr);
				(doc.body as IHTMLDOMNode).appendChild(divElem as IHTMLDOMNode);
				this.divElem = divElem;
				this.ForceHighlight();
			}

			public bool InlineHighlight(int x, int y) 
			{
				if (this.boundary.Contains(x, y) && !this.inside) 
				{
					this.divElem.style.filter = FULLALPHA;
					IEnumerator ienum = this.imageIcons.GetEnumerator();
					while (ienum.MoveNext()) 
					{
						(ienum.Current as IHTMLElement).style.filter = FULLALPHA;
					}
					this.inside = true;
				} 
				if (!this.alwaysHighlight && !this.boundary.Contains(x, y) && this.inside)
				{
					this.divElem.style.filter = INIALPHA;
					IEnumerator ienum = this.imageIcons.GetEnumerator();
					while (ienum.MoveNext()) 
					{
						(ienum.Current as IHTMLElement).style.filter = INIALPHA;
					}
					this.inside = false;
				}
				return this.boundary.Contains(x, y);
			}
			public void UpdateHighlight(IHTMLDocument2 doc) 
			{
//				int offsetTop = (doc.body as IHTMLElement2).scrollTop;
//				int offsetLeft = (doc.body as IHTMLElement2).scrollLeft;

				int minimalLeft = int.MaxValue;
				int maximalRight = int.MinValue;
				int minimalTop = int.MaxValue;
				int maximalBottom = int.MinValue;

				IEnumerator ienum = items.GetEnumerator();
				while (ienum.MoveNext()) 
				{
					IHTMLInputElement inel = (ienum.Current as ProtectedInput).Elem;
					//IHTMLRect rect = (inel as IHTMLElement2).getBoundingClientRect();
//					minimalLeft = minimalLeft > rect.left ? rect.left : minimalLeft;
//					maximalRight = maximalRight < rect.right ? rect.right : maximalRight;
//					minimalTop = minimalTop > rect.top ? rect.top : minimalTop;
//					maximalBottom = maximalBottom < rect.bottom ? rect.bottom : maximalBottom;
					Rectangle rect = this.GetInlinePosition(inel as IHTMLElement);
					minimalLeft = minimalLeft > rect.Left ? rect.Left : minimalLeft;
					maximalRight = maximalRight < rect.Right ? rect.Right : maximalRight;
					minimalTop = minimalTop > rect.Top ? rect.Top : minimalTop;
					maximalBottom = maximalBottom < rect.Bottom ? rect.Bottom : maximalBottom;
				}
				ienum = this.imageIcons.GetEnumerator();
				while (ienum.MoveNext()) 
				{
					IHTMLElement el = (IHTMLElement) ienum.Current;
//					IHTMLRect rect = (el as IHTMLElement2).getBoundingClientRect();
//					minimalLeft = minimalLeft > rect.left ? rect.left : minimalLeft;
//					maximalRight = maximalRight < rect.right ? rect.right : maximalRight;
//					minimalTop = minimalTop > rect.top ? rect.top : minimalTop;
//					maximalBottom = maximalBottom < rect.bottom ? rect.bottom : maximalBottom;
					Rectangle rect = this.GetInlinePosition(el);
					minimalLeft = minimalLeft > rect.Left ? rect.Left : minimalLeft;
					maximalRight = maximalRight < rect.Right ? rect.Right : maximalRight;
					minimalTop = minimalTop > rect.Top ? rect.Top : minimalTop;
					maximalBottom = maximalBottom < rect.Bottom ? rect.Bottom : maximalBottom;
				}
				int divX = minimalLeft - PADDING;
				divX = divX > 0 ? divX : 0;
				int divY = minimalTop - PADDING;
				divY = divY > 0 ? divY : 0;
				int divW = maximalRight + PADDING - divX;
				int divH = maximalBottom + PADDING - divY;
//				divX += offsetTop;
//				divY += offsetLeft;
				this.boundary = new Rectangle(divX, divY, divW, divH);
				string divXStr = divX.ToString() + "px";
				string divYStr = divY.ToString() + "px";
				string divWStr = divW.ToString() + "px";
				string divHStr = divH.ToString() + "px";
				string divElemStr = "<div style=\"position:absolute; left:" + divXStr + "; top:" + divYStr + 
					"; width:" + divWStr + "; height:" + divHStr + 
					"; border: " + BORDER + "; filter: " + INIALPHA + "; z-index:101\"></div>";
				IHTMLElement divElem = doc.createElement(divElemStr);
				if (this.divElem == null) 
					(doc.body as IHTMLDOMNode).appendChild(divElem as IHTMLDOMNode);
				else
					(doc.body as IHTMLDOMNode).replaceChild((divElem as IHTMLDOMNode), (this.divElem as IHTMLDOMNode));
				this.divElem = divElem;
				this.ForceHighlight();
			}
			public void ForceHighlight (bool b) 
			{
				this.alwaysHighlight = b;
				this.ForceHighlight();
			}
			private void ForceHighlight() 
			{
				if (this.alwaysHighlight) 
				{
					this.divElem.style.filter = FULLALPHA;
					IEnumerator ienum = this.imageIcons.GetEnumerator();
					while (ienum.MoveNext()) 
					{
						(ienum.Current as IHTMLElement).style.filter = FULLALPHA;
					}
				} 
				else 
				{
					this.divElem.style.filter = INIALPHA;
					IEnumerator ienum = this.imageIcons.GetEnumerator();
					while (ienum.MoveNext()) 
					{
						(ienum.Current as IHTMLElement).style.filter = INIALPHA;
					}
				}
			}
			private Rectangle GetInlinePosition(IHTMLElement el) 
			{
				int x = el.offsetLeft;
				int y = el.offsetTop;
				int w = el.offsetWidth;
				int h = el.offsetHeight;
				while (!el.offsetParent.tagName.Equals("BODY")) 
				{
					el = el.offsetParent;
					x += el.offsetLeft;
					y += el.offsetTop;
				}
				return new Rectangle(x, y, w, h);
			}
		}

		public class ProtectedInput 
		{
			IHTMLInputElement elem;
			public IHTMLInputElement Elem 
			{
				get 
				{
					return this.elem;
				}
			}
			ConstantAttributes.INPUTTYPE sensitiveType;
			public ConstantAttributes.INPUTTYPE SensitiveType 
			{
				get 
				{
					return this.sensitiveType;
				}
			}
			ConstantAttributes.ATTRIBUTE attribute;
			public ConstantAttributes.ATTRIBUTE Attribute 
			{
				get 
				{
					return this.attribute;
				}
			}

			public ProtectedInput (IHTMLInputElement e, ConstantAttributes.INPUTTYPE st, ConstantAttributes.ATTRIBUTE attr) 
			{
				this.elem = e;
				this.sensitiveType = st;
				this.attribute = attr;
			}
		}

		public ProtectedGroup(IEBrowser b)
		{
			this.browser = b;
			this.analyzer = this.browser.WebpageAnalyzer;
			this.groups = new ArrayList();
			this.focus = null;
		}

		public void ResetProtectedGroup() 
		{
			this.groups = new ArrayList();
			this.focus = null;
		}
		public virtual void FormProtectedGroup(IHTMLDocument2 doc, bool h) 
		{
			if (doc == null)
				return;

			// TODO: The HardCodeAnalyze is only working for 
			// the current prototype and the user study
			// PROPOSED SOLUTIONS:
			// 1. Heuristic Analysis of the input labels
			// 2. Analysis based on the newly-defined attributes of the inputs
			this.groups = analyzer.HardcodedAnalyze(doc);
			if (this.groups.Count == 0) 
				this.focus = null;
			else 
			{
				this.focus = this.groups[0] as ProtectedHolder;
				this.InlineAnnotation(doc);
				this.focus.ForceHighlight(h);
			}
		}
		protected void InlineAnnotation(IHTMLDocument2 doc) 
		{
			IEnumerator ienum = this.groups.GetEnumerator();
			while (ienum.MoveNext()) 
			{
				(ienum.Current as ProtectedHolder).InlineAnnotation(doc);
			}
		}
		public bool InlineHighlight(int x, int y) 
		{
			bool highlighter = false;
			IEnumerator ienum = this.groups.GetEnumerator();
			while (ienum.MoveNext()) 
			{
				bool b = (ienum.Current as ProtectedHolder).InlineHighlight(x, y);
				highlighter = highlighter | b;
			}
			return highlighter;
		}
		public void UpdateHighlight(IHTMLDocument2 doc) 
		{
			IEnumerator ienum = this.groups.GetEnumerator();
			while (ienum.MoveNext()) 
				(ienum.Current as ProtectedHolder).UpdateHighlight(doc);
		}

		public void ForceHighlight(bool b) 
		{
			if (this.focus != null)
				this.focus.ForceHighlight(b);
		}

		public WebWallet.InlineProtectionToolkit.SensitiveData.SensitiveData GetFocusedRequest() 
		{
			if (this.focus != null)
				return this.focus.Sdata;
			else
				return null;
		}

		public Rectangle GetFocusedRectangle() 
		{
			if (this.focus != null && this.focus.DivElem != null) 
			{
				IHTMLRect r = (this.focus.DivElem as IHTMLElement2).getBoundingClientRect();
				return new Rectangle(r.left, r.top, r.right-r.left, r.bottom-r.top);
			}
			else
				return AnnotationLayer.INVALIDRECT;
		}

	}
}
