using System;
using System.Drawing;

namespace PrivacyGuard.WebWallet
{
	/// <summary>
	/// The WebWalletInterface has 4 components:
	/// 1. the fixed title bar
	/// 2. the card input panel: has two sizes (one is restored and the other is max mode)
	/// 3. the card folder: has fixed size or maybe two sizes
	/// 4. the no-affordance error panel: has fixed size
	/// </summary>
	public class WebWalletInterface : System.Windows.Forms.Panel
	{
		//public const string FONTNAME = "Microsoft Sans Serif";
		public const string FONTNAME = "Tahoma";

		public const int INTERH = 10;
		public const int BORDER = 5;

		public const int TITLEY = 0;
		public const int TITLEH = 40;

		public const int INPUTFACEX = 10;
		public const int INPUTFACEY = TITLEY + TITLEH + INTERH;
		public const int INPUTFACEW = 230;
		public const int INPUTFACERH = 265;
//		public const int INPUTFACERH = 200;
//		public const int ERRORFACEH = 270;
		public const int INPUTFACEMH = IEBrowser.WEBWALLETH - INPUTFACEY - BORDER;

		public const int FOLDERX = 5;
		public const int FOLDERY = INPUTFACEY + INPUTFACERH + INTERH;
		public const int FOLDERW = 240;
		public const int FOLDERH = IEBrowser.WEBWALLETH - FOLDERY - BORDER;

//		public const int FOLDEREY = INPUTFACEY + ERRORFACEH + INTERH;
//		public const int FOLDEREH = IEBrowser.WEBWALLETH - FOLDEREY - BORDER;

		private IEBrowser browser;
		public IEBrowser Browser 
		{
			get 
			{
				return this.browser;
			}
		}

		private WebWallet.WebWalletToolkit.Titlebar titlebar;
		private WebWallet.WebWalletToolkit.DataInputInterface inputInterface;
		public WebWallet.WebWalletToolkit.DataInputInterface InputInterface 
		{
			get 
			{
				return this.inputInterface;
			}
		}
		private WebWallet.WebWalletToolkit.CardFolder cardfolder;
		public WebWallet.WebWalletToolkit.CardFolder Cardfolder 
		{
			get 
			{
				return this.cardfolder;
			}
		}
//		private WebWallet.WebWalletToolkit.ErrorInterface errorInterface;

		public WebWalletInterface(IEBrowser b)
		{
			this.browser = b;
			InitializeComponent();
		}

		private void InitializeComponent() 
		{
			this.Location = new System.Drawing.Point(IEBrowser.MAXIMIZEDX-IEBrowser.WEBWALLETW, IEBrowser.BROWSINGY);
			this.Size = new System.Drawing.Size(IEBrowser.WEBWALLETW, IEBrowser.WEBWALLETH);

			this.titlebar = new PrivacyGuard.WebWallet.WebWalletToolkit.Titlebar(this);
			this.inputInterface = new PrivacyGuard.WebWallet.WebWalletToolkit.DataInputInterface(this);
			this.cardfolder = new PrivacyGuard.WebWallet.WebWalletToolkit.CardFolder(this);
			//this.errorInterface = new PrivacyGuard.WebWallet.WebWalletToolkit.ErrorInterface(this);

			this.inputInterface.Location = new Point(INPUTFACEX, INPUTFACEY);

			this.Controls.Add(this.titlebar);
			this.Controls.Add(this.inputInterface);
			this.Controls.Add(this.cardfolder);
			//this.Controls.Add(this.errorInterface);
		}

		/// <summary>
		/// every page complete will update the WebWalletInterface
		/// 1. the card input panel is in the restored mode
		/// 2. the card folder is shown
		/// 3. the error panel is hidden
		/// </summary>
		public void UpdateDisplay() 
		{
			// if the page has request, then show the data input interface
			// else show the no affordance error interface
//			WebWallet.InlineProtectionToolkit.SensitiveData.SensitiveData rdata
//				= this.browser.ProtectedGroup.GetFocusedRequest();
//			PhishingAttacks.PhishingConfig.ATTACKTYPE at = 
//				this.browser.Phishingmonitor.GetPhishingType(this.browser.GetDomainName(true));
//			bool showError = (at == PhishingAttacks.PhishingConfig.ATTACKTYPE.FAKEWW);

//			if (rdata != null && !showError) 
//			{
//				this.inputInterface.Show();
//				this.errorInterface.Hide();
//				this.inputInterface.Size = new Size(INPUTFACEW, INPUTFACERH);
//				this.inputInterface.UpdateDisplay();
//			} 
//			else 
//			{
//				this.inputInterface.Hide();
//				this.errorInterface.Show();
//				this.errorInterface.UpdateDisplay();
//				this.inputInterface.UpdateDisplay();
//			}

			this.inputInterface.Show();
			this.inputInterface.Size = new Size(INPUTFACEW, INPUTFACERH);
			this.inputInterface.UpdateDisplay();

			// show title and the card folder;
			this.cardfolder.Show();
			this.cardfolder.UpdateDisplay();
		}

		public void Close() 
		{
			this.browser.WebWalletClose();
		}

		public void Submit() 
		{
			this.UpdateDisplay();
			//this.inputInterface.Submit(false);
		}

//		public void ShowErrorInterface() 
//		{
//			this.inputInterface.Hide();
//			this.cardfolder.Show();
//			this.cardfolder.ClearFolder();
//			this.errorInterface.Show();
//			this.errorInterface.UpdateDisplay();
//		}
//		public void HideErrorInterface() 
//		{
//			this.errorInterface.Hide();
//			this.inputInterface.Show();
//			this.inputInterface.Size = new Size(INPUTFACEW, INPUTFACERH);
//			this.cardfolder.Show();
//		}

		public void ShowSiteReport (WebWalletToolkit.DomainDescription sender) 
		{
			Rectangle rect = sender.RectangleToScreen(sender.ClientRectangle);
			Point tr = new Point(rect.Left,rect.Top);
			if (sender.Domain != null) 
			{
				WebWallet.SiteReportBox sitereport = new SiteReportBox(this.browser, sender.Domain, tr);
				sitereport.Show();
			}
		}
		public void ShowSiteComparison(WebWalletToolkit.DomainComparison sender) 
		{
			Rectangle rect = sender.RectangleToScreen(sender.ClientRectangle);
			Point tr = new Point(rect.Left,rect.Top);
			if (sender.Domain1 != null && sender.Domain2 != null) 
			{
				WebWallet.SiteReportBox sitereport = new SiteReportBox(this.browser, sender.Domain1, sender.Domain2, tr);
				sitereport.Show();
			}
		}

		public bool SaveCard() 
		{
			return this.inputInterface.SaveCard();
		}
	}
}
