using System;
using System.Drawing;
using System.Collections;
using System.Threading;

namespace PrivacyGuard.WebWallet.WebWalletToolkit
{
	/// <summary>
	/// The DataInputInterface has 6 components:
	/// 1. the card panel: fixed size and location
	///		display either new or stored card based on the request from the current page.
	/// if the card is new card, then
	/// 2. the save prompt: fixed size and location
	/// 3. the ok button: fixed size, two locations
	/// 4. the cancel button: fixed size, two locations
	/// when the ok button is pressed, then
	/// 5. the secondary interface: fixed size, two locations with one location is (0,0)
	///		display either the choosing, the confirming or the dnd direction interface
	/// </summary>
	public class DataInputInterface : System.Windows.Forms.Panel
	{
		public const int INTERH = 10;

		public const int CARDX = 0;
		public const int CARDY = 0;
		public const int CARDW = WebWalletInterface.INPUTFACEW;
		public const int CARDH = 145;

		public const int SAVEX = 0;
		public const int SAVEY = CARDY + CARDH + INTERH;
		public const int SAVEW = WebWalletInterface.INPUTFACEW;
		public const int SAVEH = 30;

		public const int OKBUTTONX = 40;
		public const int CANCELBUTTONX = 140;
		public const int BUTTONRY = SAVEY + SAVEH + INTERH;
		public const int BUTTONW = 70;
		public const int BUTTONH = 20;
		public const int BUTTONMY = WebWalletInterface.INPUTFACEMH - BUTTONH;
		

		public const int SECONDX = 0;
		public const int SECONDY = BUTTONRY;
		public const int SECONDW = WebWalletInterface.INPUTFACEW;
		public const int SECONDH = BUTTONMY - SECONDY - INTERH;

		private WebWalletInterface parentInterface;
		public WebWalletInterface ParentInterface 
		{
			get 
			{
				return this.parentInterface;
			}
		}
		private IEBrowser browser;
		public IEBrowser Browser 
		{
			get 
			{
				return this.browser;
			}
		}

		//private CardToolkit.Card dataCard;
		private System.Windows.Forms.Panel dataCard;
		public CardToolkit.Card DataCard 
		{
			get 
			{
				return (this.dataCard as CardToolkit.Card);
			}
			set 
			{
				this.dataCard = (value as System.Windows.Forms.Panel);
			}
		}
		private System.Windows.Forms.CheckBox savePrompt;
		private System.Windows.Forms.Button okButton;
		private System.Windows.Forms.Button cancelButton;
//		private System.Windows.Forms.Button yesButton;
//		private System.Windows.Forms.Button noButton;

		private System.Windows.Forms.Panel secondaryInterface;
		public enum SECONDDISPLAY {CHOOSING, CONFIRMING, CONFIRMING2, DND};

		private DnDDirectionInterface dndDirection;
		public DnDDirectionInterface DndDirection 
		{
			get 
			{
				return this.dndDirection;
			}
		}

		private ConfirmingInterface confirmingInterface;
		private ConfirmingInterface2 confirmingInterface2;
		private ChoosingInterface choosingInterface;

		private bool onlyConfirm = false;
		public bool OnlyConfirm 
		{
			get 
			{
				return this.onlyConfirm;
			}
		}

		private WebWallet.InlineProtectionToolkit.SensitiveData.SensitiveData dataSubmission;
		public WebWallet.InlineProtectionToolkit.SensitiveData.SensitiveData DataSubmission 
		{
			get 
			{
				return this.dataSubmission;
			}
		}

		private CardToolkit.CardPageBridge bridge;
		//private bool readySubmit = false;

		public DataInputInterface(WebWalletInterface p)
		{
			this.parentInterface = p;
			this.browser = p.Browser;
			this.bridge = new PrivacyGuard.WebWallet.WebWalletToolkit.CardToolkit.CardPageBridge();
			this.savePrompt = new System.Windows.Forms.CheckBox();
			this.okButton = new System.Windows.Forms.Button();
			this.cancelButton = new System.Windows.Forms.Button();
//			this.yesButton = new System.Windows.Forms.Button();
//			this.noButton = new System.Windows.Forms.Button();
			this.dataCard = new System.Windows.Forms.Panel();

			this.secondaryInterface = new System.Windows.Forms.Panel();
			this.secondaryInterface.Location = new Point(SECONDX, SECONDY);
			this.secondaryInterface.Size = new Size(SECONDW, SECONDH);
			this.secondaryInterface.Hide();
			this.choosingInterface = new ChoosingInterface(this);
			this.confirmingInterface = new ConfirmingInterface(this.parentInterface);
			this.confirmingInterface2 = new ConfirmingInterface2(this);
			this.dndDirection = new DnDDirectionInterface();
			
			this.secondaryInterface.Controls.Add(this.choosingInterface);
			this.secondaryInterface.Controls.Add(this.confirmingInterface);
			this.secondaryInterface.Controls.Add(this.confirmingInterface2);
			this.secondaryInterface.Controls.Add(this.dndDirection);

			this.Controls.Add(this.secondaryInterface);
			this.Controls.Add(this.savePrompt);
			this.Controls.Add(this.okButton);
			this.Controls.Add(this.cancelButton);
//			this.Controls.Add(this.yesButton);
//			this.Controls.Add(this.noButton);
			this.Controls.Add(this.dataCard);

			InitializeComponent();
		}

		private void InitializeComponent() 
		{
			this.Location = new Point(WebWalletInterface.INPUTFACEX, WebWalletInterface.INPUTFACEY);
			this.Size = new Size(WebWalletInterface.INPUTFACEW, WebWalletInterface.INPUTFACERH);

			/// 1. the card panel: fixed size and location
			this.dataCard.Location = new Point(CARDX, CARDY);
			this.dataCard.Size = new Size(CARDW, CARDH);

			/// 2. the save prompt: fixed size and location
			this.savePrompt.Text = "Save this new card into the Web Wallet if it is accepted by the web site.";
			this.savePrompt.Location = new Point(SAVEX, SAVEY);
			this.savePrompt.Size = new Size(SAVEW, SAVEH);
			this.savePrompt.Checked = true;

			this.okButton.Location = new Point(OKBUTTONX, BUTTONRY);
			this.okButton.Size = new Size(BUTTONW, BUTTONH);
			this.okButton.Click += new EventHandler(okButton_Click);

			this.cancelButton.Text = "Back";
			this.cancelButton.Location = new Point(CANCELBUTTONX, BUTTONRY);
			this.cancelButton.Size = new Size(BUTTONW, BUTTONH);
			this.cancelButton.Click += new EventHandler(cancelButton_Click);

//			this.yesButton.Text = "Yes";
//			this.yesButton.Location = new Point(0, BUTTONMY);
//			this.yesButton.Size = new Size(40, BUTTONH);
//			this.yesButton.Click += new EventHandler(yesButton_Click);
//			this.noButton.Text = "No";
//			this.noButton.Location = new Point(60, BUTTONMY);
//			this.noButton.Size = new Size(40, BUTTONH);
//			this.noButton.Click += new EventHandler(noButton_Click);
//
//			this.yesButton.Hide();
//			this.noButton.Hide();

			this.dataCard.Hide();
			this.savePrompt.Hide();
			this.okButton.Hide();
			this.cancelButton.Hide();
			this.secondaryInterface.Hide();
		}

		public void UpdateDisplay() 
		{
//			this.readySubmit = false;
			this.bridge.Reset();

			this.secondaryInterface.Hide();

			if (this.browser.Phishingmonitor.GetPhishingType(this.browser.GetDomainName(true))
				== PhishingAttacks.PhishingConfig.ATTACKTYPE.FAKEWW)
				dataSubmission = null;
			else
				dataSubmission = this.browser.ProtectedGroup.GetFocusedRequest();
			if (dataSubmission != null) 
			{
				if (dataSubmission is WebWallet.InlineProtectionToolkit.SensitiveData.LoginToken) 
				{
					this.Controls.Remove(this.dataCard);
					this.dataCard = null;
					//					if (this.Controls.Contains(this.dataCard))
					//						this.Controls.Remove(this.dataCard);
					this.dataCard = FormLoginCard(dataSubmission as WebWallet.InlineProtectionToolkit.SensitiveData.LoginToken);
					(this.dataCard as CardToolkit.Card).Initialization();
					this.Controls.Add(this.dataCard);
					if (this.dataCard is CardToolkit.NewLoginCard) 
						ShowNewCard(false);
					else 
						ShowStoredCard();
				}
			} 
			else 
			{
//				string domain = this.browser.GetDomainName(false);
//				this.parentInterface.Cardfolder.LogincardFolder.SetDefaultSelection(domain);
			}
		}

		private CardToolkit.Card FormLoginCard(WebWallet.InlineProtectionToolkit.SensitiveData.LoginToken t) 
		{
			CardToolkit.Card c = null;
			BackEndImpl.SecureStorage.LoginToken st;
			string d = this.browser.GetDomainName(false);
			//st = this.browser.SecureStorage.ContainsLoginRecord(t.Domain, t.GetAuthScope());
			st = this.browser.SecureStorage.ContainsLoginRecord(d, BackEndImpl.SecureStorage.DEFAULTSCOPE);
			if (st == null) 
				c = new CardToolkit.NewLoginCard(
					dataSubmission as WebWallet.InlineProtectionToolkit.SensitiveData.LoginToken, 
					this.bridge,
					this.browser);
			else 
				c = new CardToolkit.StoredLoginCard(st, d,
					dataSubmission as WebWallet.InlineProtectionToolkit.SensitiveData.LoginToken, 
					this.bridge,
					this.browser);
			return c;
		}

		public void SwitchNewLoginCard() 
		{
//			this.readySubmit = false;
			this.bridge.Reset();

			this.secondaryInterface.Hide();

			this.Controls.Remove(this.dataCard);
			this.dataCard = null;

//			if (this.Controls.Contains(this.dataCard))
//				this.Controls.Remove(this.dataCard);

			if (this.dataSubmission != null) 
				this.dataCard =  new CardToolkit.NewLoginCard(
					dataSubmission as WebWallet.InlineProtectionToolkit.SensitiveData.LoginToken, 
					this.bridge,
					this.browser);
			else
				this.dataCard =  new CardToolkit.NewLoginCard(
					null,
					this.bridge,
					this.browser);
			
			(this.dataCard as CardToolkit.Card).Initialization();

			this.Controls.Add(this.dataCard);
			ShowNewCard(false);
		}

		public void SwitchStoredLoginCard(string domain) 
		{
			if ((this.dataCard != null) &&
				(this.dataCard is CardToolkit.StoredLoginCard) && 
				(this.dataCard as CardToolkit.StoredLoginCard).GetCardDomain().Equals(domain)) 
			{
				if (!this.dataCard.Visible)
					ShowStoredCard();
				return;
			}

//			this.readySubmit = false;
			this.bridge.Reset();

			this.secondaryInterface.Hide();

			this.Controls.Remove(this.dataCard);
			this.dataCard = null;

//			if (this.Controls.Contains(this.dataCard))
//				this.Controls.Remove(this.dataCard);

			if (dataSubmission != null)
				this.dataCard = new CardToolkit.StoredLoginCard
					(this.browser.SecureStorage.ContainsLoginRecord(domain, BackEndImpl.SecureStorage.DEFAULTSCOPE), 
					domain,
					dataSubmission as WebWallet.InlineProtectionToolkit.SensitiveData.LoginToken, 
					this.bridge,
					this.browser);
			else
				this.dataCard = new CardToolkit.StoredLoginCard
					(this.browser.SecureStorage.ContainsLoginRecord(domain, BackEndImpl.SecureStorage.DEFAULTSCOPE), 
					domain,
					null,
					this.bridge,
					this.browser);

			(this.dataCard as CardToolkit.Card).Initialization();

			this.Controls.Add(this.dataCard);

			ShowStoredCard();
		}

		private void yesButton_Click(object sender, System.EventArgs e) 
		{
			this.ShowSecondaryInterface(DataInputInterface.SECONDDISPLAY.CONFIRMING, true);
		}
		private void noButton_Click(object sender, System.EventArgs e) 
		{
			this.ShowSecondaryInterface(DataInputInterface.SECONDDISPLAY.CONFIRMING2, true);
		}
		private void okButton_Click(object sender, System.EventArgs e) 
		{
			string text = (sender as System.Windows.Forms.Button).Text;
			if (text.Equals("Continue")) 
			{
				// when the ok button displays "continue"
				// 1. the card panel displays a new card
				// 2. the input is in the restored mode
				// when the "continue" button is clicked
				// 1. the data input panel is in the max mode
				// 2. the card folder is hidden
				// 3. the choosing panel shown up 
				// 4. the 2 buttons move
				
				this.ShowSecondaryInterface(SECONDDISPLAY.CHOOSING, true);
			} 
			else if (text.Equals("Fill in")) 
			{
				if (!this.secondaryInterface.Visible) 
					FillinNewCard();
				else if (this.choosingInterface.Visible) 
				{
					if (!this.choosingInterface.SelectionMatch())
						this.ShowSecondaryInterface(DataInputInterface.SECONDDISPLAY.CONFIRMING, true);
					else if (this.choosingInterface.SelectionTrusted())
						FillinNewCard();
					else
						this.ShowSecondaryInterface(DataInputInterface.SECONDDISPLAY.CONFIRMING2, true);
				}					
				else if (this.confirmingInterface2.Visible)
					FillinNewCard();
			} 
			//			else if (text.Equals("Back")) 
			//			{
			//				// when the ok button displays "back"
			//				// 1. the input is in the max mode
			//				// 2. the confirming interface is shown
			//				// when the "back" button is clicked
			//				// 1. the confirming interface is hidden
			//				// 2. the choosing interface is shown
			//				ShowConfirmation(false);
			//			}
			//		
		}
		private void cancelButton_Click(object sender, System.EventArgs e) 
		{
			if (this.choosingInterface.Visible) 
			{
				this.HideSecondaryInterface();
				this.ShowNewCard(false);
			}
			else if (this.confirmingInterface.Visible) 
			{
				this.ShowSecondaryInterface(SECONDDISPLAY.CHOOSING, false);
			} 
			else if (this.confirmingInterface2.Visible) 
			{
				this.ShowSecondaryInterface(SECONDDISPLAY.CHOOSING, false);
			} 
			else if (this.dndDirection.Visible) 
			{
//				this.parentInterface.ShowErrorInterface();
				this.HideSecondaryInterface();
				this.ShowNewCard(false);
			}
		}
		public void Submit() 
		{
			if (this.dataSubmission != null && !this.bridge.HasSubmit) 
			{
//				if (this.readySubmit) 
//				{
					Thread nt = new Thread(new ThreadStart(this.AnimateSubmit));
					nt.Start();
//				}
			}
		}

		public void FillinNewCard() 
		{
			if (this.dataSubmission != null) 
			{
				if (!this.secondaryInterface.Visible) 
					this.Submit();
				else 
				{
					this.Submit();
					this.HideSecondaryInterface();
					this.ShowNewCard(true);
				}				
			}				
			else 
			{
				this.ShowSecondaryInterface(SECONDDISPLAY.DND, true);
			}
		}

		private void AnimateSubmit() 
		{
			Thread.Sleep(100);
			Rectangle startRect = this.dataCard.RectangleToScreen(this.dataCard.ClientRectangle);
			Rectangle refRect = this.browser.GetBrowsingWindowToScreen();
			Rectangle relativeRect = this.browser.ProtectedGroup.GetFocusedRectangle();
			Rectangle endRect = new Rectangle(refRect.Left+relativeRect.Left, 
				refRect.Top+relativeRect.Top, relativeRect.Width, relativeRect.Height);
			this.browser.AnnotationLayer.DisplayDataFilling(startRect, endRect);
			this.bridge.Submit();
		}

		public void ShowSecondaryInterface(DataInputInterface.SECONDDISPLAY sd, bool update) 
		{
			this.secondaryInterface.Show();
			this.parentInterface.Cardfolder.Hide();
			this.Size = new Size(this.Width, WebWalletInterface.INPUTFACEMH);
			switch (sd) 
			{
				case SECONDDISPLAY.CHOOSING:
					this.choosingInterface.Show();
					this.confirmingInterface.Hide();
					this.confirmingInterface2.Hide();
					this.dndDirection.Hide();
					this.okButton.Show();
					this.cancelButton.Show();
//					this.yesButton.Show();
//					this.noButton.Show();
					if (update) 
					{
						this.choosingInterface.UpdateDisplay(true);
						this.okButton.Location = new Point(OKBUTTONX, BUTTONMY);
						this.okButton.Text = "Fill in";
						this.okButton.Enabled = false;
						this.cancelButton.Location = new Point(CANCELBUTTONX, BUTTONMY);
						this.cancelButton.Text = "Cancel";
						this.cancelButton.Show();

						//this.yesButton.Enabled = false;
					}
					break;

				case SECONDDISPLAY.CONFIRMING:
					this.choosingInterface.Hide();
					this.confirmingInterface.Show();
					this.confirmingInterface2.Hide();
					this.dndDirection.Hide();
					this.confirmingInterface.UpdateDisplay(
						this.choosingInterface.GetSelectedEntry().Domainname,
						this.choosingInterface.GetActualEntry().Domainname);
					this.okButton.Hide();
					this.cancelButton.Location = new Point(CANCELBUTTONX, BUTTONMY);
					this.cancelButton.Text = "Back";
					this.cancelButton.Show();
//					this.yesButton.Hide();
//					this.noButton.Hide();
					break;

				case SECONDDISPLAY.CONFIRMING2:
					this.choosingInterface.Hide();
					this.confirmingInterface.Hide();
					this.confirmingInterface2.Show();
					this.dndDirection.Hide();
					this.confirmingInterface2.UpdateDisplay();
					this.okButton.Show();
					this.okButton.Text = "Fill in";
					this.cancelButton.Location = new Point(CANCELBUTTONX, BUTTONMY);
					this.cancelButton.Text = "Back";
					this.cancelButton.Show();
//					this.yesButton.Hide();
//					this.noButton.Hide();
					break;

				case SECONDDISPLAY.DND:
					this.choosingInterface.Hide();
					this.confirmingInterface.Hide();
					this.confirmingInterface2.Hide();
					this.dndDirection.Show();
					this.okButton.Hide();
					this.cancelButton.Location = new Point(CANCELBUTTONX, BUTTONMY);
					this.cancelButton.Text = "Cancel";
					this.cancelButton.Show();
					break;

				default:
					this.choosingInterface.Hide();
					this.confirmingInterface.Hide();
					this.confirmingInterface2.Hide();
					this.dndDirection.Hide();
					this.okButton.Hide();
					this.cancelButton.Hide();
					break;
			} 
		} 

		public void HideSecondaryInterface() 
		{
			this.secondaryInterface.Hide();
			this.Size = new Size(this.Width, WebWalletInterface.INPUTFACERH);
			this.parentInterface.Cardfolder.Show();
		}

		public void ShowSecondaryInterfaceAsOnlyConfirm(string intend, string actual) 
		{
			this.onlyConfirm = true;
			this.secondaryInterface.Show();
			this.dataCard.Hide();
			this.savePrompt.Hide();
			this.okButton.Hide();
			this.cancelButton.Hide();
			this.Location = new Point(0, this.Location.Y);
			this.Size = new Size(250, WebWalletInterface.INPUTFACERH);
			this.secondaryInterface.Location = new Point(0, 0);
			this.secondaryInterface.Size = this.Size;
			this.choosingInterface.Hide();
			this.confirmingInterface.Size = this.secondaryInterface.Size;
			this.confirmingInterface.Show();
			this.dndDirection.Hide();
			this.confirmingInterface.UpdateDisplay(intend, actual);
			this.parentInterface.Cardfolder.Show();
		}

		public void RestoreSecondaryInterfaceAsOnlyConfirm() 
		{
			this.onlyConfirm = false;
			this.Location = new Point(WebWalletInterface.INPUTFACEX, this.Location.Y);
			this.Size = new Size(WebWalletInterface.INPUTFACEW, WebWalletInterface.INPUTFACERH);
			this.secondaryInterface.Location = new Point(SECONDX, SECONDY);
			this.secondaryInterface.Size = new Size(SECONDW, SECONDH);
			this.confirmingInterface.Size = this.secondaryInterface.Size;
		}

//		private void ShowChoosingInterface() 
//		{
//			// when the "continue" button is clicked
//			// 1. the data input panel is in the max mode
//			// 2. the card folder is hidden
//			// 3. the choosing panel shown up 
//			// 4. the 2 buttons move
//			this.parentInterface.Cardfolder.Hide();
//			this.Size = new Size(this.Width, WebWalletInterface.INPUTFACEMH);
//			this.choosingInterface.Show();
//			this.choosingInterface.UpdateDisplay();
//			this.okButton.Location = new Point(OKBUTTONX, BUTTONMY);
//			this.okButton.Text = "Submit";
//			this.okButton.Enabled = false;
//			this.okButton.Show();
//			//this.okButton.Hide();
//			this.cancelButton.Location = new Point(CANCELBUTTONX, BUTTONMY);
//			this.cancelButton.Show();
//
////			this.parentInterface.HideCardFolder();
////			if (!this.Controls.Contains(this.choosingInterface))
////				this.Controls.Add(this.choosingInterface);
////			this.Size = new Size(this.Width, WebWalletInterface.INPUTFACEMH);
////			this.choosingInterface.UpdateDisplay();
////			PrivacyGuardUtils.WebWalletUtil.RemoveIfNecessary(this.Controls, this.okButton);
//////			this.okButton.Location = new Point(OKBUTTONX, BUTTONMY);
//////			this.okButton.Text = "Submit";
////			this.cancelButton.Location = new Point(CANCELBUTTONX, BUTTONMY);
//		}

//		public void HideChoosingInterfaceWithDnDDirection() 
//		{
//			this.choosingInterface.Hide();
//			this.confirmingInterface.Hide();
//			this.Size = new Size(this.Width, WebWalletInterface.INPUTFACEMH);
//			this.okButton.Location = new Point(OKBUTTONX, BUTTONMY);
//			this.okButton.Text = "Back";
//			this.cancelButton.Location = new Point(CANCELBUTTONX, BUTTONMY);
//			this.okButton.Show();
//			this.cancelButton.Show();
//			this.dndDirection.Show();
//			this.parentInterface.Cardfolder.Hide();
//		}

//		public void HideChoosingInterface() 
//		{
//			// 1. the input panel is in the restored mode
//			// 2. the choosing and confirming interfaces are hidden
//			// 3. the card folder is shown
//			// 4. two buttons move
//			this.choosingInterface.Hide();
//			this.confirmingInterface.Hide();
//			this.Size = new Size(this.Width, WebWalletInterface.INPUTFACERH);
//			this.okButton.Show();
//			this.cancelButton.Show();
//			this.okButton.Location = new Point(OKBUTTONX, BUTTONRY);
//			this.okButton.Text = "Continue";
//			this.cancelButton.Location = new Point(CANCELBUTTONX, BUTTONRY);
//			this.parentInterface.Cardfolder.Show();
//
////			if (this.Controls.Contains(this.choosingInterface))
////				this.Controls.Remove(this.choosingInterface);
////			if (this.Controls.Contains(this.confirmingInterface))
////				this.Controls.Remove(this.confirmingInterface);
////			this.Size = new Size(this.Width, WebWalletInterface.INPUTFACERH);
////			if (this.dataCard is CardToolkit.NewLoginCard) 
////			{
////				PrivacyGuardUtils.WebWalletUtil.AddIfNecessary(this.Controls, this.okButton);
////				this.okButton.Location = new Point(OKBUTTONX, BUTTONRY);
////				this.okButton.Text = "Continue";
////				this.cancelButton.Location = new Point(CANCELBUTTONX, BUTTONRY);
////			}
////			this.parentInterface.ShowCardFolder();
//		}

//		public void ShowConfirmation(bool b) 
//		{
//			if (b) 
//			{
//				// 1. the choosing interface is hidden
//				// 2. the confirming interface is shown
//
//				this.choosingInterface.Hide();
//				this.confirmingInterface.Show();
//				this.confirmingInterface.UpdateDisplay(
//					this.choosingInterface.GetSelectedEntry().Domainname,
//					this.choosingInterface.GetActualEntry().Domainname);
//				this.okButton.Show();
//				this.okButton.Text = "Back";
//
////				PrivacyGuardUtils.WebWalletUtil.RemoveIfNecessary(this.Controls, this.choosingInterface);
////				PrivacyGuardUtils.WebWalletUtil.AddIfNecessary(this.Controls, this.confirmingInterface);
//////				if (this.Controls.Contains(this.choosingInterface))
//////					this.Controls.Remove(this.choosingInterface);
//////				if (!this.Controls.Contains(this.confirmingInterface))
//////					this.Controls.Add(this.confirmingInterface);
////				this.confirmingInterface.UpdateDisplay(
////					this.choosingInterface.GetSelectedEntry().Domainname,
////					this.choosingInterface.GetActualEntry().Domainname);
////				PrivacyGuardUtils.WebWalletUtil.AddIfNecessary(this.Controls, this.okButton);
////				this.okButton.Location = new Point(OKBUTTONX, BUTTONMY);
////				this.okButton.Text = "Back";
//			} 
//			else 
//			{
//				// 1. the confirming interface is hidden
//				// 2. the choosing interface is shown
//				this.confirmingInterface.Hide();
//				this.choosingInterface.Show();
//				this.okButton.Hide();
//
////				if (this.Controls.Contains(this.confirmingInterface))
////					this.Controls.Remove(this.confirmingInterface);
////				if (!this.Controls.Contains(this.choosingInterface))
////					this.Controls.Add(this.choosingInterface);
////				PrivacyGuardUtils.WebWalletUtil.RemoveIfNecessary(this.Controls, this.okButton);
////				//this.okButton.Text = "Submit";
//			}
//		}

//		public void EnableYesButton() 
//		{
//			this.yesButton.Enabled = true;
//		}

		public void EnableOKButton() 
		{
			this.okButton.Enabled = true;
		}
		public void ShowNewCard(bool after) 
		{
//			this.parentInterface.HideErrorInterface();
			this.dataCard.Show();
			this.savePrompt.Show();
			this.okButton.Location = new Point(OKBUTTONX, BUTTONRY);
			this.okButton.Enabled = true;
			this.okButton.Show();
			this.cancelButton.Location = new Point(CANCELBUTTONX, BUTTONRY);
			this.cancelButton.Hide();
			if ((this.dataCard as CardToolkit.Card).IsTrusted && 
				InHistory((this.dataCard as CardToolkit.Card).GetCardDomain()))
				this.okButton.Text = "Fill in";
			else
				this.okButton.Text = "Continue";
			if (!after)
				this.bridge.ClearPage();
		}
		private void ShowStoredCard() 
		{
//			this.parentInterface.HideErrorInterface();
			this.dataCard.Show();
			this.savePrompt.Hide();
			this.okButton.Location = new Point(OKBUTTONX, BUTTONRY);
			this.okButton.Enabled = true;
			this.okButton.Hide();
			this.cancelButton.Location = new Point(CANCELBUTTONX, BUTTONRY);
			this.cancelButton.Hide();
			//this.readySubmit = true;

			if (this.dataSubmission != null) 
				this.HideSecondaryInterface();
			else 
			{
				this.ShowSecondaryInterface(SECONDDISPLAY.DND, true);
				this.cancelButton.Show();
			}

			if (this.parentInterface.Visible)
				this.Submit();
		}

		public void SwitchConflictedStoredCard(string domain) 
		{
			this.SwitchStoredLoginCard(domain);
			this.savePrompt.Show();
		}

		public bool SaveCard() 
		{
			return this.savePrompt.Checked;
		}

		public bool InHistory(string domain) 
		{
			return this.browser.History.InHistory(domain);
		}
	}
}
