using System;
using System.Collections;
using System.Drawing;
using System.Windows.Forms;
using mshtml;

namespace PrivacyGuard.WebWallet.WebWalletToolkit
{
	/// <summary>
	/// Summary description for RecipientArea.
	/// </summary>
	public class RecipientArea : System.Windows.Forms.Panel
	{
		private IEBrowser browser;
		private ArrayList entries;

//		public const int AREAW = 270;
//		public const int AREAH = 270;
		public const int ENTRYX = 0;
		public const int ENTRYY = 2;

		private RecipientEntry selected;
		public RecipientEntry Selected 
		{
			get 
			{
				return this.selected;
			}
		}
		private RecipientEntry current;
		public RecipientEntry Current
		{
			get 
			{
				return this.current;
			}
		}

		public class RecipientEntry : System.Windows.Forms.Panel 
		{
			public const string FONTNAME = "Tahoma";
			public const int MAXWIDTH = WebWalletInterface.INPUTFACEW-17;
			public const int DOMX = 20;
			public const int DESX = 20;
			public const int DOMY = 0;
			public const int HUNIT = 15;
			public const int RATY = 2;
			public const int RATW = 65;
			public const int RATH = 12;
			public const int RATX = MAXWIDTH-RATW;
//			public const int MAXDOMWIDTH = MAXWIDTH-DOMX-RATW;
			public const int MAXDOMWIDTH = MAXWIDTH-DOMX;
			public const int MAXDESWIDTH = MAXWIDTH-DESX;

			private System.Windows.Forms.RadioButton radiobutton;
			public System.Windows.Forms.RadioButton Radiobutton 
			{
				get 
				{
					return this.radiobutton;
				}
			}

			private System.Windows.Forms.Label domain;
			private string domainname;
			public string Domainname 
			{
				get 
				{
					return this.domainname;
				}
			}

			private DomainDescription description;
			private string despstr;

//			private System.Windows.Forms.PictureBox rating;
//			private string imageuri;

			private BackEndImpl.ReputationEvaluator.ReputationRecord record;

			public RecipientEntry(BackEndImpl.ReputationEvaluator.ReputationRecord rr) 
			{
				this.record = rr;
				this.domainname = this.record.Domain;
				this.despstr = this.record.Description;
//				this.imageuri = this.record.GetRatingImage();

				InitializeComponent();
			}

			private void InitializeComponent() 
			{
				this.radiobutton = new System.Windows.Forms.RadioButton();
				this.radiobutton.Size = new Size(10, 10);
				this.radiobutton.Text = "";
				this.radiobutton.Click += new EventHandler(entry_Click);

				this.domain = new System.Windows.Forms.Label();
				this.domain.Location = new Point(DOMX, DOMY);
				this.domain.Text = this.domainname;
				this.domain.Font = new Font(FONTNAME, this.domain.Font.Size, FontStyle.Bold);
				int preferW = this.domain.PreferredWidth;
				int hu = (int) Math.Ceiling((double)preferW/(double)MAXDOMWIDTH);
				this.domain.Size = new Size(MAXDOMWIDTH, HUNIT*hu);
				this.domain.Click += new EventHandler(entry_Click);
				this.domain.MouseMove += new System.Windows.Forms.MouseEventHandler(entry_MouseMove);

				this.description = new DomainDescription(this.domainname);
				this.description.Location = new Point(DESX, this.domain.Height);
				this.description.Text = this.despstr;
//				if (!this.record.IsTrustedSite()) 
//				{
//					this.description.LinkColor = Color.Red;
//					this.description.Font = new Font(FONTNAME, this.description.Font.Size, FontStyle.Italic|FontStyle.Bold);
//				} else
				this.description.Font = new Font(FONTNAME, this.description.Font.Size, FontStyle.Italic);
				preferW = this.description.PreferredWidth;
				hu = (int)Math.Ceiling((double)preferW/(double)MAXDESWIDTH);
				this.description.Size = new Size(MAXDESWIDTH, HUNIT*hu);
				this.description.LinkClicked += new LinkLabelLinkClickedEventHandler(description_LinkClicked);

//				this.rating = new System.Windows.Forms.PictureBox();
//				this.rating.Location = new Point(RATX, RATY);
//				this.rating.Size = new Size(RATW, RATH);
//				this.rating.Image = Image.FromFile(this.imageuri);
//				this.rating.Click += new EventHandler(entry_Click);
//				this.rating.MouseMove += new System.Windows.Forms.MouseEventHandler(entry_MouseMove);

				int actualh = this.domain.Height+this.description.Height;
				this.radiobutton.Location = new Point(5, (actualh-10)/2);
				this.Size = new Size(MAXWIDTH, actualh);

				this.Controls.Add(this.radiobutton);
				this.Controls.Add(this.domain);
				this.Controls.Add(this.description);
//				this.Controls.Add(this.rating);

				//this.BorderStyle = BorderStyle.FixedSingle;
			}

			private void entry_Click(object sender, System.EventArgs e) 
			{
				(this.Parent as RecipientArea).SelectEntry(this);
			}
			private void entry_MouseMove(object sender, System.Windows.Forms.MouseEventArgs e) 
			{
				Cursor.Current = Cursors.Hand;
			}
			private void description_LinkClicked (object sender, System.Windows.Forms.LinkLabelLinkClickedEventArgs e) 
			{
				(this.Parent as RecipientArea).browser.Webwallet.ShowSiteReport(sender as DomainDescription);
			}
			public bool IsTrusted() 
			{
				return this.record.IsTrustedSite();
			}
			//			public RecipientEntry CopyEntry() 
//			{
//				return new RecipientEntry(this.domainname, this.despstr, this.imageuri);
//			}
//			public void HideRadioButton() 
//			{
//				this.radiobutton.Hide();
//				this.domain.Location = new Point(this.domain.Left-15, this.domain.Top);
//				this.description.Location = new Point(this.description.Left-15, this.description.Top);
//				this.rating.Location = new Point(this.rating.Left-15, this.rating.Top);
//				this.Size = new Size(this.Width-15, this.Height);
//			}
		}

		public RecipientArea(IEBrowser b)
		{
			this.browser = b;
			this.entries = new ArrayList();
			this.BorderStyle = BorderStyle.FixedSingle;
		}

		public void UpdateDisplay() 
		{
			this.Controls.Clear();
			this.selected = null;
			string domainname = this.browser.GetDomainName(false);
			this.current = 
				new RecipientEntry(this.browser.ReputationEvaluator.GetReputationRecord(domainname));
			ArrayList al = new ArrayList();
			bool includedomain = false;
			Object[] ha = this.browser.History.ToArray();
			for (int i = 0; i < ha.Length; i++) 
			{
				string dom = ((BackEndImpl.History.HistoryEntry) ha[i]).Domain;
				if (domainname.Equals(dom))
					includedomain = true;
				BackEndImpl.ReputationEvaluator.ReputationRecord rr = 
					this.browser.ReputationEvaluator.GetReputationRecord(dom);
				al.Add(rr);
			}
			if (!includedomain) 
			{
				BackEndImpl.ReputationEvaluator.ReputationRecord rr = 
					this.browser.ReputationEvaluator.GetReputationRecord(domainname);
				al.Add(rr);
			}

			BackEndImpl.ReputationEvaluator.SortReputationRecords(ref al);

			this.BackColor = Color.White;
			this.AutoScroll = true;

			int y = ENTRYY;
			bool addbkcolor = false;
			for (int i = 0; i < al.Count; i++) 
			{
				BackEndImpl.ReputationEvaluator.ReputationRecord record = (BackEndImpl.ReputationEvaluator.ReputationRecord) al[i];
				RecipientEntry re = new RecipientEntry(record);
				this.entries.Add(re);
				re.Location = new Point(ENTRYX, y);
				re.BackColor = addbkcolor ? Color.LightGray : Color.White;
				this.Controls.Add(re);
				y += re.Height;
				addbkcolor = !addbkcolor;
			}
		}

		public void SelectEntry(RecipientEntry e) 
		{
			//(this.Parent.Parent as ChoosingInterface).RemoveWarning();
//			(this.Parent.Parent.Parent.Parent as DataInputInterface).EnableYesButton();
			(this.Parent.Parent.Parent as DataInputInterface).EnableOKButton();
			this.selected = e;
			IEnumerator ienum = this.entries.GetEnumerator();
			while (ienum.MoveNext()) 
			{
				RecipientEntry c = (RecipientEntry) ienum.Current;
				c.Radiobutton.Checked = (c.Equals(e));
			}
//			ChoosingInterface ci = this.Parent.Parent as ChoosingInterface;
//			//ci.SelectionChange();
//			if (ci.ReadySubmit()) 
//			{
//				ci.InputInterface.Submit(true);
//				ci.InputInterface.HideChoosingInterface();
//			}
		}

//		protected override void OnPaint(System.Windows.Forms.PaintEventArgs pe) 
//		{
//			Graphics g = pe.Graphics;
//			Pen pn = new Pen(Color.Black);
//			Point start = new Point(0,0);
//			Point end = new Point(this.Width,0);
//			g.DrawLine(pn, start, end);
//		}
	}
}
